def lookup_lists(list1, list2, match_check=lambda x, y: x == y):  # left outer join
    """
    Joins two lists based on a match condition
    :param list1: Base list
    :param list2: Lookup list
    :param match_check: Lambda with 2 arguments to match the elements between the two lists
    :return: A list containing a dictionary with "base" and "lookup"
    """
    result = []
    for iter1 in list1:
        matching_iter2 = None
        for iter2 in list2:
            if match_check(iter1, iter2):
                matching_iter2 = iter2
                break
        result.append({'base': iter1, 'lookup': matching_iter2})
    return result


def compare_lists(list1, list2, equal_check=lambda x, y: x == y):
    """
    Compares two lists and returns the diff
    :param list1: First list
    :param list2: Second list
    :param equal_check: Optional, Lambda with 2 arguments to compare the elements of the two lists
    :return: A dictionary containing the keys +, -, =
    """
    result = {
        '=': [],
        '+': [],
        '-': []
    }
    for iter1 in list1:
        is_iter1_found_in_iter2 = False
        for iter2 in list2:
            if equal_check(iter1, iter2):
                is_iter1_found_in_iter2 = True
                break
        if is_iter1_found_in_iter2:
            result['='].append(iter1)
        else:
            result['+'].append(iter1)
    for iter2 in list2:
        is_iter2_found_in_iter1 = False
        for iter1 in list1:
            if equal_check(iter1, iter2):
                is_iter2_found_in_iter1 = True
                break
        if not is_iter2_found_in_iter1:
            result['-'].append(iter2)
    return result


def list_first(a_list):
    if a_list is None:
        return None

    tmp_list = list(a_list)  # In case we have a filter result instead of an actual list
    if len(tmp_list) == 0:
        return None

    return tmp_list[0]




from datetime import datetime, timedelta, date


def days_before_at_this_time(x, ref_date=datetime.now()):
    return ref_date - timedelta(days=x)


def days_ago_at_this_time(x, ref_date=datetime.now()):
    return days_before_at_this_time(x, ref_date)


def days_after_at_this_time(x, ref_date=datetime.now()):
    return ref_date + timedelta(days=x)


def days_since_at_this_time(x, ref_date=datetime.now()):
    return days_after_at_this_time(x, ref_date)


def tomorrow_at_this_time():
    return datetime.now() + timedelta(days=1)


def yesterday_at_this_time():
    return datetime.now() - timedelta(days=1)


def days_before(x, ref_date=date.today()):
    return ref_date - timedelta(days=x)


# Alias for days_before
def days_ago(x):
    return days_before(x)


def days_after(x, ref_date=date.today()):
    return ref_date + timedelta(days=x)


# Alias for days_after
def days_since(x, ref_date=date.today()):
    return days_after(x, ref_date)


def tomorrow():
    return date.today() + timedelta(days=1)


def yesterday():
    return date.today() - timedelta(days=1)


def hours_before(x, ref_date=datetime.now()):
    return ref_date - timedelta(hours=x)


# Alias for hours_before
def hours_ago(x):
    return hours_before(x)


def hours_after(x, ref_date=datetime.now()):
    return ref_date + timedelta(hours=x)


# Alias for hours_after
def hours_since(x, ref_date=datetime.now()):
    return hours_after(x, ref_date)


def minutes_before(x, ref_date=datetime.now()):
    return ref_date - timedelta(minutes=x)


# Alias for minutes_before
def minutes_ago(x):
    return minutes_before(x)


def minutes_after(x, ref_date=datetime.now()):
    return ref_date + timedelta(minutes=x)


# Alias for minutes_after
def minutes_since(x, ref_date=datetime.now()):
    return minutes_after(x, ref_date)


def seconds_before(x, ref_date=datetime.now()):
    return ref_date - timedelta(minutes=x)


# Alias for seconds_before
def seconds_ago(x):
    return seconds_before(x)


def seconds_after(x, ref_date=datetime.now()):
    return ref_date + timedelta(minutes=x)


# Alias for seconds_after
def seconds_since(x, ref_date=datetime.now()):
    return seconds_after(x, ref_date)


def weeks_before(x, ref_date=date.today()):
    return ref_date - timedelta(weeks=x)


def weeks_before_at_this_time(x, ref_date=datetime.now()):
    return ref_date - timedelta(weeks=x)


def weeks_ago_at_this_time(x, ref_date=datetime.now()):
    return weeks_before_at_this_time(x, ref_date)


def weeks_after_at_this_time(x, ref_date=datetime.now()):
    return ref_date + timedelta(weeks=x)


def weeks_since_at_this_time(x, ref_date=datetime.now()):
    return weeks_after_at_this_time(x, ref_date)


# Alias for seconds_before
def weeks_ago(x):
    return weeks_before(x)


def weeks_after(x, ref_date=date.today()):
    return ref_date + timedelta(weeks=x)


# Alias for seconds_after
def weeks_since(x, ref_date=date.today()):
    return weeks_after(x, ref_date)


# Internal method to support the month_* functions
def month_add(x, ref_date=date.today()):
    years_to_add = int((ref_date.month + x) / 12)
    months_to_set = (ref_date.month + x) % 12
    if type(ref_date) is datetime:
        return datetime(ref_date.year + years_to_add, months_to_set, ref_date.day, ref_date.hour, ref_date.minute, ref_date.second, ref_date.microsecond)
    elif type(ref_date) is date:
        return date(ref_date.year + years_to_add, months_to_set, ref_date.day)


def months_before(x, ref_date=date.today()):
    return month_add(-1*x, ref_date)


# Alias for months_before
def months_ago(x):
    return months_before(x)


def months_before_at_this_time(x, ref_date=datetime.now()):
    return month_add(-1*x, ref_date)


def months_ago_at_this_time(x, ref_date=datetime.now()):
    return months_before_at_this_time(x, ref_date)


def months_after_at_this_time(x, ref_date=datetime.now()):
    return month_add(x, ref_date)


def months_since_at_this_time(x, ref_date=datetime.now()):
    return months_after_at_this_time(x, ref_date)


def months_after(x, ref_date=date.today()):
    return month_add(x, ref_date)


# Alias for months_after
def months_since(x, ref_date=date.today()):
    return months_after(x, ref_date)


def years_before(x, ref_date=date.today()):
    if type(ref_date) is datetime:
        return datetime(ref_date.year - x, ref_date.month, ref_date.day, ref_date.hour, ref_date.minute, ref_date.second, ref_date.microsecond)
    elif type(ref_date) is date:
        return date(ref_date.year - x, ref_date.month, ref_date.day)


# Alias for years_before
def years_ago(x):
    return years_before(x)


def years_after(x, ref_date=date.today()):
    return years_before(-x, ref_date)


# Alias for years_after
def years_since(x, ref_date=date.today()):
    return years_after(x, ref_date)


def years_before_at_this_time(x, ref_date=datetime.now()):
    return years_before(x, ref_date)


def years_ago_at_this_time(x, ref_date=datetime.now()):
    return years_before_at_this_time(x, ref_date)


def years_after_at_this_time(x, ref_date=datetime.now()):
    return years_after(x, ref_date)


def years_since_at_this_time(x, ref_date=datetime.now()):
    return years_after_at_this_time(x, ref_date)




from datetime import datetime

def exec_time(func_arg):
    """
    Prints the execution time for a function
    :param func_arg:
    :return:
    """
    def inner_exec_time():
        dt1 = datetime.now()
        func_arg()
        dt2 = datetime.now()
        print(f"{func_arg.__name__} ran in {dt2 - dt1}")
    return inner_exec_time


class CacheOrLambda:
    my_cache = {}
    my_lambda = None

    def __init__(self, a_lambda):
        self.my_lambda = a_lambda

    def fetch(self, a_key):
        retVal = None
        if a_key in self.my_cache:
            retVal = self.my_cache[a_key]
        else:
            retVal = self.my_lambda(a_key)
            self.my_cache[a_key] = retVal
        return retVal



from .lists_helpers import compare_lists
from .lists_helpers import lookup_lists
from .lists_helpers import list_first

from .date_helpers import days_before
from .date_helpers import days_ago
from .date_helpers import days_after
from .date_helpers import days_since
from .date_helpers import tomorrow
from .date_helpers import yesterday
from .date_helpers import hours_before
from .date_helpers import hours_ago
from .date_helpers import hours_after
from .date_helpers import hours_since
from .date_helpers import minutes_before
from .date_helpers import minutes_ago
from .date_helpers import minutes_after
from .date_helpers import minutes_since
from .date_helpers import seconds_before
from .date_helpers import seconds_ago
from .date_helpers import seconds_after
from .date_helpers import seconds_since
from .date_helpers import weeks_before
from .date_helpers import weeks_ago
from .date_helpers import weeks_after
from .date_helpers import weeks_since
from .date_helpers import months_before
from .date_helpers import months_ago
from .date_helpers import months_after
from .date_helpers import months_since
from .date_helpers import years_before
from .date_helpers import years_ago
from .date_helpers import years_after
from .date_helpers import years_since

from .dict_helpers import make_dict_path
from .dict_helpers import set_dict_path
from .dict_helpers import get_dict_path
from .dict_helpers import deep_copy_dict

from .string_helpers import string_or_default

from .function_helpers import CacheOrLambda
from .function_helpers import exec_time


def string_or_default(primary_value, secondary_value):
    retVal = primary_value
    if retVal is None or retVal.strip() == '':
        retVal = secondary_value
    return retVal
    # return secondary_value if (primary_value is None or primary_value.strip()) == '' else primary_value



def make_dict_path(a_dict, a_path):
    set_dict_path(a_dict, a_path, {})


def set_dict_path(a_dict, a_path, a_val):
    curr_obj = a_dict
    path_len = len(a_path)
    counter = 0
    for iter_attr in a_path:
        counter += 1
        if counter == path_len:
            curr_obj[iter_attr] = a_val
        if not iter_attr in curr_obj:
            curr_obj[iter_attr] = {}
        curr_obj = curr_obj[iter_attr]


def get_dict_path(a_dict, a_path):
    curr_obj = a_dict
    for iter_attr in a_path:
        if iter_attr in curr_obj:
            curr_obj = curr_obj[iter_attr]
        else:
            return None
    return curr_obj


def deep_copy_dict(a_src, a_dest, overwrite=False):
    for k, v in a_src.items():
        if isinstance(v, dict) and k in a_dest:
            deep_copy_dict(a_src[k], a_dest[k], overwrite)
        else:
            if ((overwrite == True) or (not k in a_dest)):
                a_dest[k] = v



from setuptools import setup, find_packages

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='pappardelle',
    version='0.10',
    long_description_content_type='text/markdown',
    long_description=long_description,
    packages=find_packages(),
    install_requires=[
    ]
)
